#define NS_BUILD_32_LIKE_64 1
#include <CoreFoundation/CoreFoundation.h>
#include <CoreServices/CoreServices.h>
#include <QuickLook/QuickLook.h>

#import "ACImage.h"

// #define debug NSLog
#define debug(...)

OSStatus TSGeneratePreviewForURL(void *thisInterface, QLPreviewRequestRef preview, CFURLRef url, CFStringRef contentTypeUTI, CFDictionaryRef options) {
    
    @autoreleasepool {
        
        // Create and read the document file
        ACImage *img = [ACImage imageWithURL:(NSURL *)url];
        
        NSSize canvasSize = [img canvasSize];
        
        CGContextRef cgContext = QLPreviewRequestCreateContext(preview, canvasSize, true, nil);
        if(cgContext) {
            
            [img drawInContext:cgContext ofSize:canvasSize];
            
            QLPreviewRequestFlushContext(preview, cgContext);
            
            CFRelease(cgContext);
        }
    }
    
    return noErr;
}

void TSCancelPreviewGeneration(void* thisInterface, QLPreviewRequestRef preview) {
    // implement only if supported
}



OSStatus TSGenerateThumbnailForURL(void *thisInterface, QLThumbnailRequestRef thumbnail, CFURLRef url, CFStringRef contentTypeUTI, CFDictionaryRef options, CGSize maxSize) {
    
    debug(@"%s:%d", __FUNCTION__, __LINE__);
    
    @autoreleasepool {
        
        debug(@"%s:%d", __FUNCTION__, __LINE__);
        // Create and read the document file
        ACImage *img = [ACImage imageWithURL:(NSURL *)url];
        
        debug(@"%s:%d", __FUNCTION__, __LINE__);
        BOOL hasThumb = [img hasThumbnail];
        
        debug(@"%s:%d", __FUNCTION__, __LINE__);
        NSSize canvasSize = hasThumb ? [img thumbnailSize] : [img canvasSize];
        
        debug(@"%s:%d", __FUNCTION__, __LINE__);
        NSSize imageSizeToDraw = canvasSize;
        
        debug(@"%s:%d", __FUNCTION__, __LINE__);
        if (maxSize.width < imageSizeToDraw.width || maxSize.height < imageSizeToDraw.height) {
            
            if (imageSizeToDraw.width > imageSizeToDraw.height) {
                double r = maxSize.width / imageSizeToDraw.width;
                imageSizeToDraw.height = floor(imageSizeToDraw.height * r);
                imageSizeToDraw.width = maxSize.width;
            }
            else {
                double r = maxSize.height / imageSizeToDraw.height;
                imageSizeToDraw.width = floor(imageSizeToDraw.width * r);
                imageSizeToDraw.height = maxSize.height;
            }
        }
        
        debug(@"%s:%d", __FUNCTION__, __LINE__);
        CGContextRef cgContext = QLThumbnailRequestCreateContext(thumbnail, imageSizeToDraw, YES, NULL);
        if (cgContext) {
            
            if (hasThumb) {
                [img drawThumbnailInContext:cgContext ofSize:imageSizeToDraw];
            }
            else {
                [img drawInContext:cgContext ofSize:imageSizeToDraw];
            }
            
            QLThumbnailRequestFlushContext(thumbnail, cgContext);
            
            CFRelease(cgContext);
        }
        else {
            NSLog(@"Can't create a context for %@.", url);
        }
        debug(@"%s:%d", __FUNCTION__, __LINE__);
            
    }
    
    debug(@"%s:%d", __FUNCTION__, __LINE__);
    
    return noErr;
}

void TSCancelThumbnailGeneration(void* thisInterface, QLThumbnailRequestRef thumbnail) {
    // implement only if supported
}

